@@ -1,12 +1,4 @@
-"""
-    Sphinx
-    ~~~~~~
-
-    The Sphinx documentation toolchain.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""The Sphinx documentation toolchain."""
 
 # Keep this file executable as-is in Python 3!
 # (Otherwise getting the version out of it from setup.py is impossible.)
@@ -19,11 +11,6 @@ from subprocess import PIPE
 
 from .deprecation import RemovedInNextVersionWarning
 
-if False:
-    # For type annotation
-    from typing import Any  # NOQA
-
-
 # by default, all DeprecationWarning under sphinx package will be emit.
 # Users can avoid this by using environment variable: PYTHONWARNINGS=
 if 'PYTHONWARNINGS' not in os.environ:
@@ -31,9 +18,11 @@ if 'PYTHONWARNINGS' not in os.environ:
 # docutils.io using mode='rU' for open
 warnings.filterwarnings('ignore', "'U' mode is deprecated",
                         DeprecationWarning, module='docutils.io')
+warnings.filterwarnings('ignore', 'The frontend.Option class .*',
+                        DeprecationWarning, module='docutils.frontend')
 
-__version__ = '3.5.3'
-__released__ = '3.5.3'  # used when Sphinx builds its own docs
+__version__ = '5.1.1'
+__released__ = '5.1.1'  # used when Sphinx builds its own docs
 
 #: Version info for better programmatic use.
 #:
@@ -43,7 +32,7 @@ __released__ = '3.5.3'  # used when Sphinx builds its own docs
 #:
 #: .. versionadded:: 1.2
 #:    Before version 1.2, check the string ``sphinx.__version__``.
-version_info = (3, 5, 3, 'final', 0)
+version_info = (5, 1, 1, 'final', 0)
 
 package_dir = path.abspath(path.dirname(__file__))
 
@@ -57,8 +46,8 @@ if __version__.endswith('+'):
     try:
         ret = subprocess.run(['git', 'show', '-s', '--pretty=format:%h'],
                              cwd=package_dir,
-                             stdout=PIPE, stderr=PIPE)
+                             stdout=PIPE, stderr=PIPE, encoding='ascii')
         if ret.stdout:
-            __display_version__ += '/' + ret.stdout.decode('ascii').strip()
+            __display_version__ += '/' + ret.stdout.strip()
     except Exception:
         pass
