@@ -1,12 +1,4 @@
-"""
-    sphinx.ext.napoleon
-    ~~~~~~~~~~~~~~~~~~~
-
-    Support for NumPy and Google style docstrings.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+"""Support for NumPy and Google style docstrings."""
 
 from typing import Any, Dict, List
 
@@ -49,7 +41,7 @@ class Config:
     .. _Google style:
        https://google.github.io/styleguide/pyguide.html
     .. _NumPy style:
-       https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt
+       https://numpydoc.readthedocs.io/en/latest/format.html#docstring-standard
 
     Attributes
     ----------
@@ -288,7 +280,7 @@ class Config:
     }
 
     def __init__(self, **settings: Any) -> None:
-        for name, (default, rebuild) in self._config_values.items():
+        for name, (default, _rebuild) in self._config_values.items():
             setattr(self, name, default)
         for name, value in settings.items():
             setattr(self, name, value)
@@ -309,11 +301,11 @@ def setup(app: Sphinx) -> Dict[str, Any]:
     See Also
     --------
     `The Sphinx documentation on Extensions
-    <http://sphinx-doc.org/extensions.html>`_
+    <https://www.sphinx-doc.org/extensions.html>`_
 
-    `The Extension Tutorial <http://sphinx-doc.org/extdev/tutorial.html>`_
+    `The Extension Tutorial <https://www.sphinx-doc.org/extdev/tutorial.html>`_
 
-    `The Extension API <http://sphinx-doc.org/extdev/appapi.html>`_
+    `The Extension API <https://www.sphinx-doc.org/extdev/appapi.html>`_
 
     """
     if not isinstance(app, Sphinx):
@@ -444,10 +436,10 @@ def _skip_member(app: Sphinx, what: str, name: str, obj: Any,
 
     """
     has_doc = getattr(obj, '__doc__', False)
-    is_member = (what == 'class' or what == 'exception' or what == 'module')
+    is_member = what in ('class', 'exception', 'module')
     if name != '__weakref__' and has_doc and is_member:
         cls_is_owner = False
-        if what == 'class' or what == 'exception':
+        if what in ('class', 'exception'):
             qualname = getattr(obj, '__qualname__', '')
             cls_path, _, _ = qualname.rpartition('.')
             if cls_path:
